<?php
/**
* Extension: SimpleSAMLphp Authentication
* MediaWiki version: tested with 1.16.*, 1.17.0
* SimpleSAMLphp version 1.6.*
* Plugin version: 1.4.1
* Last update: 2011-09-14
* Developer: Drazenko Celjak <drazenko.celjak@srce.hr>
* University Computing Centre, University of Zagreb
*
*
* LocalSettings.php code:

# simpleSAMLphp authentication 
$simplesamlphp_basedir = '/your/path/to/simplesamlphp'; // simplesamlphp path
$samlVersion = 'saml2'; // saml version; allowed values are: 'saml2' or 'shib13'
$forceSSO = false; // forces user to login
$simpleSAMLphpUserAutoCreate = true; // auto create a user that doesn't exist 

# attributes used by SimpleSAMLphp authentication (if array first value is taken):
$usernameAttribute  = 'urn:mace:dir:attribute-def:eduPersonPrincipalName';
$mailAttribute      = 'urn:mace:dir:attribute-def:mail';
$givenNameAttribute = 'urn:mace:dir:attribute-def:givenName';
$surnameAttribute   = 'urn:mace:dir:attribute-def:sn';

# setting up user permissions 
$wgGroupPermissions['*']['createaccount'] = false;
$wgGroupPermissions['*']['read']          = true;
$wgGroupPermissions['*']['edit']          = false;
$wgGroupPermissions['*']['createpage']    = false;
$wgGroupPermissions['*']['createtalk']    = false;


require_once($simplesamlphp_basedir . '/lib/_autoload.php');
require_once('extensions/SimpleSAMLphpAuth.php');


// authorization (OPTIONAL)
// $wgHooks['simpleSAMLphpAuthorization'][] = 'customAuthHookFunction';
// function customAuthHookFunction($attributes) {
	// example of custom authorization implementation: http://developer.aaiedu.hr/faq/mediawiki-simplesamlphp.php
// }


* Changelog 
* v1.4
* support for SimpleSAMLphp 1.6.* and higher
* v1.3
* fixed single log-out bug (thanks to Gijsbert Sliedrecht)
* added authorization hook (simpleSAMLphpAuthorization)
*


* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along
* with this program. If not, see <http://www.gnu.org/licenses/>.
*/

require_once('includes/AuthPlugin.php');

class SimpleSAMLphpAuthPlugin extends AuthPlugin {
			
	/**
	 * Check whether there exists a user account with the given name.
	 * The name will be normalized to MediaWiki's requirements, so
	 * you might need to munge it (for instance, for lowercase initial
	 * letters).
	 *
	 * @param string $username
	 * @return bool
	 * @access public
	 */
	function userExists( $username ) {
		return true;
	}
	/**
	 * Check if a username+password pair is a valid login.
	 * The name will be normalized to MediaWiki's requirements, so
	 * you might need to munge it (for instance, for lowercase initial
	 * letters).
	 *
	 * @param string $username
	 * @param string $password
	 * @return bool
	 * @access public
	 */
	function authenticate( $username, $password) {
		return true;
	}
	
	/**
	 * When a user logs in, optionally fill in preferences and such.
	 * For instance, you might pull the email address or real name from the
	 * external user database.
	 *
	 * The User object is passed by reference so it can be modified; don't
	 * forget the & on your function declaration.
	 *
	 * @param User $user
	 * @public
	 */
	function updateUser( &$user ) {
		global $simplesaml_email;
		global $simplesaml_RN;
		
		wfDebug("updateUser: " . $user->getName() . "; {$simplesaml_email}; {$simplesaml_RN}");
		
		if ($simplesaml_email !=  null) {
			$user->setEmail($simplesaml_email);
			$user->mEmailAuthenticated = wfTimestampNow();
		}		
		if ($simplesaml_RN != '') {
			$user->setRealName($simplesaml_RN);
			$user->setOption('nickname',$simplesaml_RN);
		}
		return true;
	}

	/**
	 * Return true if the wiki should create a new local account automatically
	 * when asked to login a user who doesn't exist locally but does in the
	 * external auth database.
	 *
	 * If you don't automatically create accounts, you must still create
	 * accounts in some way. It's not possible to authenticate without
	 * a local account.
	 *
	 * This is just a question, and shouldn't perform any actions.
	 *
	 * @return bool
	 * @access public
	 */
	function autoCreate() {
		global $simpleSAMLphpUserAutoCreate;
		return ($simpleSAMLphpUserAutoCreate);
	}

	/**
	 * Can users change their passwords?
	 *
	 * @return bool
	 */
	function allowPasswordChange() {
		return false;
	}	
	
	/**
	 * Return true to prevent logins that don't authenticate here from being
	 * checked against the local database's password fields.
	 *
	 * This is just a question, and shouldn't perform any actions.
	 *
	 * @return bool
	 * @public
	 */
	function strict() {
		return true;
	}

	/**
	 * Check if a user should authenticate locally if the global authentication fails.
	 * If either this or strict() returns true, local authentication is not used.
	 *
	 * @param $username String: username.
	 * @return bool
	 * @public
	 */
	function strictUserAuth( $username ) {
		return true;
	}

	/**
	 * When creating a user account, optionally fill in preferences and such.
	 * For instance, you might pull the email address or real name from the
	 * external user database.
	 *
	 * The User object is passed by reference so it can be modified; don't
	 * forget the & on your function declaration.
	 *
	 * @param $user User object.
	 * @param $autocreate bool True if user is being autocreated on login
	 * @public
	 */
	function initUser( &$user, $autocreate=false ) {		
		global $simplesaml_email;
		global $simplesaml_RN;
		
		wfDebug("initUser: " . $user->getName() . "; {$simplesaml_email}; {$simplesaml_RN}");
		
		if ($simplesaml_email !=  null) {
			$user->setEmail($simplesaml_email);
			$user->mEmailAuthenticated = wfTimestampNow();
		}		
		if ($simplesaml_RN != '') {
			$user->setRealName($simplesaml_RN);
			$user->setOption('nickname',$simplesaml_RN);
		}		
		return true;
	}
		
} // class SimpleSAMLphpAuthPlugin








# Hooks

function ssp_userLoadFromSession($user, &$result) {
	global $wgAuth;
	global $simplesaml_UN;
	global $wgHooks;
	global $wgContLang;
	global $wgUser;
	global $forceSSO;	
		
	// prevent loop
	foreach ($wgHooks['UserLoadFromSession'] as $key => $value) {
		if($value == 'ssp_userLoadFromSession')
			unset($wgHooks['UserLoadFromSession'][$key]);
    }

	$username = $wgContLang->ucfirst($simplesaml_UN);
	
	// is user already logged in?
	$wgUser = User::newFromSession();
	$wgUser->load();		
	
	if ($wgUser->isLoggedIn() && $wgUser->getName() == $username) {
		// user is logged in - do nothing		
	} else {	
		// not logged in; user exists in db?
		$user = User::newFromName($username);						
		if ($user->getID() == 0) {
			// no
			if ($wgAuth->autoCreate()) {
				$wgAuth->initUser($user, true);
				$user->addToDatabase();
			} else {
				// user does not exist in db, autoCreate is false 
				if ($forceSSO) {
					die('Access denied (user account creation disabled).');
				} else {
					// sso session, wiki user account creation disabled
					return true;
				}
			}
		} else {
			// user exists - update after each login; optional 2 lines
			$wgAuth->updateUser($user); 
			$user->saveSettings();
		}
		
		$user->setCookies();
		wfSetupSession();
		$wgUser = $user;
	}
	return true;		
}

function ssp_logOutHook(&$user) {
	$as = new SimpleSAML_Auth_Simple('default-sp');
	if ($as->isAuthenticated()){
		$as->logout();
	}
	if ($forceSSO) {
		ssp_redirect();
	}
}

function ssp_hideLogOutLink(&$personal_urls, $title) {
	if (isset($personal_urls['anonlogin']))
		unset($personal_urls['anonlogin']);
	if (isset($personal_urls['logout']))
		unset($personal_urls['logout']);
    return true;
}

function ssp_loginHookFunction($template) {
	$as = new SimpleSAML_Auth_Simple('default-sp');
	if (!$as->isAuthenticated()){
		$as->requireAuth();
	} else {
		ssp_redirect();
	}
}

function ssp_getAttributeValue($attributes, $name) {
	if (isset($attributes[$name])) {
		if (is_array($attributes[$name]))
			return $attributes[$name][0];
		else 
			return $attributes[$name];
	}
	return '';
}

function ssp_redirect(){
	global $wgScriptPath;
	$path = ($wgScriptPath == '') ? '/' : $wgScriptPath;
	header("Location: ".$path);
	exit();
}





// Disable caching
$wgCachePages = false;
// Characters to prevent during new account creations.
$wgInvalidUsernameCharacters = '#';
// Character used as a delimiter when testing for interwiki userrights
$wgUserrightsInterwikiDelimiter = '#';
// disable login form
$wgHooks['UserLoginForm'][] = 'ssp_loginHookFunction'; 






$as = new SimpleSAML_Auth_Simple('default-sp');

if( $as->isAuthenticated()){

	$attributes = $as->getAttributes();

	$wgAuth = new SimpleSAMLphpAuthPlugin();
					
	/*
	 * You should beware of possible namespace collisions, it is best to chose
	 * something that will not violate MW's usual restrictions on characters
	 */	 
	$simplesaml_UN = ssp_getAttributeValue($attributes, $usernameAttribute);
	$simplesaml_RN = ssp_getAttributeValue($attributes, $givenNameAttribute);
	$simplesaml_RN .= ' ' . ssp_getAttributeValue($attributes, $surnameAttribute);
	$simplesaml_email = ssp_getAttributeValue($attributes, $mailAttribute);
	
	// Check the apache error log to see that the correct user attributes are reckognized.
	// error_log("USER-LOGIN [$simplesaml_RN, $simplesaml_email, $simplesaml_UN]");	
	if (!isset($simplesaml_UN) || strlen($simplesaml_UN) == 0) {
		echo "Username is not set. Check attribute names.";
		exit();
	} 

	// handle custom authorization hook - if there is any
	if (isset($wgHooks['simpleSAMLphpAuthorization'])) {
		require_once( "$IP/includes/Hooks.php" );
		wfRunHooks( 'simpleSAMLphpAuthorization', array(&$attributes) );
	}
	
	$wgHooks['UserLoadFromSession'][] = 'ssp_userLoadFromSession';	
	if ($samlVersion == 'saml2') {
		// $wgHooks['PersonalUrls'][] = 'singleLogOutLinkAdd';
		$wgHooks['UserLogoutComplete'][] = 'ssp_logOutHook';
	} else {
		$wgHooks['PersonalUrls'][] = 'ssp_hideLogOutLink';	
	}
	
} else {

	if ($forceSSO) {
		// Dont allow access to wiki without sso session:
		$as->requireAuth();
	}

}


$wgExtensionCredits['other'][] = array(
   'name' => 'SimpleSAMLphpAuthPlugin',
   'version' => '1.4.1',
   'author' => 'Draženko Celjak',
   'url' => 'http://developer.aaiedu.hr/faq/mediawiki-simplesamlphp.php',
   'description' => 'SSO authentication using SimpleSAMLphp'
);
?>